using Microsoft.AspNetCore.Mvc;
using System.Drawing;
using GrapeCity.Documents.Excel;
using GrapeCity.Documents.Excel.Drawing;

namespace BTC_Chart_WebService.Controllers
{
    [ApiController]
    [Route("[controller]")]
    public class BTCChartController : ControllerBase
    {
        //    private static readonly string[] Summaries = new[]
        //    {
        //    "Freezing", "Bracing", "Chilly", "Cool", "Mild", "Warm", "Balmy", "Hot", "Sweltering", "Scorching"
        //};


        // Get the CSV data from the AlphaVantage web service
        private string GetCsvData()
        {
            string csv;
            string API_KEY = "YOUR_API_KEY";
            string QUERY_URL = $"https://www.alphavantage.co/query?function=DIGITAL_CURRENCY_MONTHLY&symbol=BTC&market=USD&apikey={API_KEY}&datatype=csv";
            Uri queryUri = new Uri(QUERY_URL);
            using (HttpClient client = new HttpClient())
            {
                Task<string> t = client.GetStringAsync(queryUri);
                while (!t.IsCompleted)
                    t.Wait();
                csv = t.Result;
            }
            return csv;
        }

        private readonly ILogger<BTCChartController> _logger;

        public BTCChartController(ILogger<BTCChartController> logger)
        {
            _logger = logger;
        }

        [HttpGet(Name = "GetBTC-USDChartWorkbook")]
        //public IEnumerable<WeatherForecast> Get()
        public FileContentResult Get()
        {
            //return Enumerable.Range(1, 5).Select(index => new WeatherForecast
            //{
            //    Date = DateTime.Now.AddDays(index),
            //    TemperatureC = Random.Shared.Next(-20, 55),
            //    Summary = Summaries[Random.Shared.Next(Summaries.Length)]
            //})
            //.ToArray();

            // first get CSV data
            string csv = GetCsvData();

            // create new workbook
            IWorkbook wbk = new Workbook();
            // open CSV data in GcExcel using MemoryStream
            using (Stream s = new MemoryStream())
            {   // convert to byte array using UTF8 encoding
                byte[] arr = System.Text.Encoding.UTF8.GetBytes(csv.ToCharArray());
                s.Write(arr);
                s.Seek(0, SeekOrigin.Begin);
                // open CSV in workbook
                wbk.Open(s, OpenFileFormat.Csv);
            }

            // get the worksheet with the CSV data
            IWorksheet wks = wbk.Worksheets[0];
            // move volume to column B, before open/high/low/close
            // it works better to create the StockVOHLC chart with
            // the series in the correct order (date-volume-open-high-low-close)
            wks.Range["B:B"].Insert();
            wks.Range["K:K"].Copy(wks.Range["B:B"]);
            wks.Range["K:K"].Delete();
            // get the range containing the CSV data
            IRange used = wks.UsedRange;
            // create a new Table for the CSV data named "BTC_Monthly"
            ITable tbl = wks.Tables.Add(used, true);
            tbl.Name = "BTC_Monthly";
            // auto-fit the columns in the table to show all the cell values
            used.AutoFit();
            // add the StockVOHLC chart shape over the table (the table is completely beneath the chart)
            IShape shape = wks.Shapes.AddChart(ChartType.StockVOHLC, 0, 0, used.Width, used.Height);
            // get the IChart from the shape
            IChart chart = shape.Chart;
            // set the chart title
            chart.ChartTitle.Text = "BitCoin Monthly Open-High-Low-Close-Volume";
            // add the chart series (the first 6 columns in the table)
            chart.SeriesCollection.Add(wks.Range[0, 0, used.RowCount, 6], RowCol.Columns, true, true);

            //change category axis base to months
            IAxis categoryAxis = chart.Axes.Item(AxisType.Category);
            categoryAxis.BaseUnit = TimeUnit.Months;
            categoryAxis.TickLabels.Orientation = 45;
            categoryAxis.TickLabels.NumberFormat = "d/m/yyyy";

            // add trend line for Volume series in Blue
            ITrendline voltrend = chart.SeriesCollection[0].Trendlines.Add();
            voltrend.Name = "3 Month Moving Avg VOL";
            voltrend.Type = TrendlineType.MovingAvg;
            voltrend.Period = 3;
            voltrend.Format.Line.Color.RGB = Color.Blue;
            voltrend.Format.Line.DashStyle = LineDashStyle.RoundDot;
            // add trend line for High series in Green
            ITrendline hightrend = chart.SeriesCollection[2].Trendlines.Add();
            hightrend.Name = "3 Month Moving Avg HIGH";
            hightrend.Type = TrendlineType.MovingAvg;
            hightrend.Period = 3;
            hightrend.Format.Line.Color.RGB = Color.Green;
            hightrend.Format.Line.DashStyle = LineDashStyle.RoundDot;
            // add trend line for Low series in Red
            ITrendline lowtrend = chart.SeriesCollection[3].Trendlines.Add();
            lowtrend.Name = "3 Month Moving Avg LOW";
            lowtrend.Type = TrendlineType.MovingAvg;
            lowtrend.Period = 3;
            lowtrend.Format.Line.Color.RGB = Color.Red;
            lowtrend.Format.Line.DashStyle = LineDashStyle.RoundDot;

            


            // Save Workbook to XLSX and return from web service as "BTC_Chart.xlsx"
            using (MemoryStream ms = new MemoryStream())
            {
                wbk.Save(ms, SaveFileFormat.Xlsx);
                ms.Seek(0, SeekOrigin.Begin);
                byte[] bytes = ms.ToArray();
                return File(bytes, "application/vnd.openxmlformats-officedocument.spreadsheetml.sheet", "BTC_Chart.xlsx");
            }
        } // Get()
    }//BTCChartController
}